import React, { useState, useRef, useEffect } from 'react'
import { Input, Modal } from 'antd'
import { CompassOutlined, CloseOutlined } from '@ant-design/icons'
import { ArcGISSceneMap, ClickQuery, geomUtils } from '@wisdom-map/arcgismap'
import './index.less'

let geometrystr = ''

const Device = ({ value, onChange, schema }) => {

  const mapConfig = window.globalConfig.mapsettings?.layers || null
  const { placeholder, disabled } = schema
  const [visible, setVisible] = useState(false)
  const layersConifg = { layers: mapConfig }
  let clickref = useRef(null)
  let view = useRef(null)

  const close = () => {
    clickref.current.closetip()
  }

  const confirm = (geometry) => {
    const { x, y } = geometry
    if (x && y) {
      onChange(`${x},${y}`)
      setVisible(false)
    }
  }

  const tip = (data) => {
    if (!Array.isArray(data)) return null
    const { displayFieldName, fields, attributes, geometry } = data[0]

    if (geometry) {
      geometrystr = `${geometry.x},${geometry.y}`
    }

    useEffect(() => {
      if (clickref.current) {
        view.current.goTo({
          center: geomUtils.toGeometry({ x: geometry.x, y: geometry.y + 400 }),
          zoom: 15
        })
      }
    }, [data])

    return (
      <div className='tip'>
        <div className='tip-header'>
          <div>{displayFieldName}</div>
          <span className='tip-close' onClick={() => close()}><CloseOutlined /></span>
        </div>
        <div className='tip-content'>
          {
            fields?.map(v => {
              if (v.editTypeMobile !== '隐藏') {
                return (
                  <div className='tip-list'>
                    <div className='tip-list-left'>{v.name || v.alias}</div>
                    <div className='tip-list-right'>{attributes[v.name]}</div>
                  </div>
                )
              }
              return null
            })
          }
        </div>
        <div className='tip-footer'>
          <span className='confirm' onClick={() => confirm(geometry)}>确定</span>
          <div className='triangle'></div>
          <div className='angle'></div>
        </div>
      </div>
    );
  };

  const getMapInfo = (mapObj, Map) => {
    setTimeout(() => {
      if (mapObj) {
        view.current = mapObj
      }
      let arr = value && value.split(',')
      let obj = (arr.length === 2) ? { point: { x: Number(arr[0]), y: Number(arr[1]) } } : {}
      clickref.current = new ClickQuery({
        view: view.current,
        tipcontent: tip,
        offsetX: -150,
        offsetY: -310,
        continuous: true,
        ...obj
      })
    }, 500)
  }

  const iconClick = () => {
    if (!schema.disabled) {
      setVisible(true)
    }
  }

  const onOk = () => {
    onChange(geometrystr)
    setVisible(false)
  }

  const onCancel = () => {
    setVisible(false)
  }

  return (
    <div>
      <Input
        disabled={disabled}
        value={value}
        placeholder={placeholder}
        addonAfter={disabled ? null : <CompassOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={iconClick} />}
        style={{ width: '100%' }}
      />
      <Modal
        visible={visible}
        title='设备选择'
        width={'80%'}
        onOk={onOk}
        onCancel={onCancel}
        cancelText={'取消'}
        okText={'确定'}
        bodyStyle={{ height: 600, overflowY: 'auto', position: 'relative' }}
        destroyOnClose={true}
      >
        <ArcGISSceneMap
          getMapInfo={getMapInfo}
          widgets={[]}
          config={layersConifg}
        />
      </Modal>
    </div>
  )
}

export default Device