/*
 ** 人员选择器 多选
 ** create by ChenLong on 2021/12/21
 ** 功能路径：src\core\components\staffSelect\staffSelect.js
 ** 菜单参数列表：*变量名*(变量说明，数据类型，是否必填，取值范围)
 *  ** 形态：【人员选择器】
 ** 现阶段仅支持机构
 ** 显示名称，存储id，多个id以英文逗分隔
 ** 配置：全部 [.n]                   返回全部的机构及人员列表
        机构名称 [.n]               返回设定的机构及人员列表
        (机构名称,机构名称 ...) [.n] 返回设定的某几个机构人员列表
  * Tip：未剔除原有逻辑，需要后续阅读代码，删除无效代码
 ** 更新日志：
 ** edit by ChenLong on 2021/12/21: *****修改内容*****
 ** edit by ChenLong on 2021/12/28: ***** 支持原有配置（存储人名，显示人名）*****
 ** edit by ChenLong on 2021/12/28: ***** 存储方式的更改会导致控件的表现异常 *****
 ** edit by ChenLong on 2022/06/09: ***** 存在使用Modal单独引入后，数据状态始终存在的问题（但是在台账组件中，暂时未发现异常；该问题与使用方式有关）。变更部分逻辑， *****
 */
import React, { useState, useMemo, useEffect } from 'react';
import { Modal, Spin, Input, Checkbox } from 'antd';
import { UserAddOutlined } from '@ant-design/icons';
import Empty from '@wisdom-components/empty';
import { getUserListForRole } from '../../../../apis/process';
import './staffSelect.less';

const { Search } = Input;

const MultipleStaffSelect = ({ value, onChange, schema }) => {

  let val = value || schema.default

  const [visible, setVisible] = useState(false);
  const [defaultPerson, setDefaultPerson] = useState([]);
  const [personArr, setPersonArr] = useState([]);
  const [searchVal, setSearchVal] = useState('');
  const [loading, setLoading] = useState(true);
  // 将所选人员存储到一个对象中,key `person_${index}`;
  const [allPersonList, setAllPersonList] = useState({});
  // 全选
  // const [indeterminate, setIndeterminate] = useState(false);
  const [allState, setAllState] = useState(false);
  const [checkAll, setCheckAll] = useState({});
  // 配置清单
  const ouConfigArray = schema.config.split('.');
  const isAll = ['全部', '全部ID'].includes(ouConfigArray[0]);
  const isMultiple = !isAll && ouConfigArray[0].includes(',') && ouConfigArray.includes('(') && ouConfigArray.includes(')'); // (机构名,机构名)

  const storeType = useMemo(() => {
    return isAll && ouConfigArray[0] === '全部ID' ? 'userID' : 'userName'
  }, [isAll, ouConfigArray])

  const userNames = useMemo(() => {
    if (storeType === 'userName') return value
    let userList = personArr.reduce((total, current) => total.concat(current.userList), [])
    let array = []
    userList?.forEach(v => {
      if (value.includes(v.userID)) {
        array.push(v.userName)
      }
    })
    return array.join(',')
  }, [personArr, value, storeType])

  /**
   *  @Description: 通过判断storeType来决定返回的【已选择人员数组】是存储人名还是ID；
   *  @Params: 参数描述
   *  @Date: 2021/12/28
   *  @Author: ChenLong
   * */
  const returnCheckedUserIDArray = (values, data = personArr) => {
    let _checkedArray = values.split(',');
    let _allPersonList = {};
    data.forEach((item, index) => {
      let _tempArray = [];
      item.userList.forEach((obj) => {
        if (_checkedArray.includes(obj[storeType])) {
          _tempArray.push(obj[storeType]);
        }
      });
      _allPersonList[`person_${index}`] = _tempArray;
    });
    return _allPersonList;
  };
  const checkInitNames = (IDs) => {
    return IDs.split(',');
  };
  const getData = () => {
    setLoading(true)
    let ouConfig = ouConfigArray[0]; // 机构名的配置
    getUserListForRole().then(res => {
      if (res.code === 0) {
        const arr = [];
        res.data.forEach(item => {
          // 全部机构
          if (isAll) {
            arr.push(item);
          } else if (isMultiple) {
            // 多机构
            let _ouConfigArray = ouConfig.replace(/\(|\)/g, '').split(',');
            if (_ouConfigArray.includes(item.ouName)) arr.push(item);
          } else {
            // 单机构
            let _ouConfigArray = ouConfig.replace(/\(|\)/g, '');
            if (_ouConfigArray.includes(item.ouName)) arr.push(item);
          }
        });
        setDefaultPerson(arr);
        setPersonArr(arr);
        let _allPersonList = returnCheckedUserIDArray(value, res.data);
        setAllPersonList(_allPersonList);
        // 初始化时，先创建checkAll对象，避免后续第一次打开人员选择器，
        let _checkAll = { ...checkAll };
        arr.forEach((item, index) => {
          _checkAll[`person_${index}`] = false;
        })
        setCheckAll(_checkAll);
      } else {
        onChange('请求错误');
      }
      setLoading(false);
    });
  };
  const setDataStatus = () => {
    setCheckAll({});
    setAllPersonList({});
    setAllState(false);
  };
  const onFocus = () => {
    setVisible(true)
    onChange(val)
    if (!personArr.length) {
      getData()
    }
  };
  const onOk = () => {
    setVisible(false);
    setSearchVal('');
    // 合并allPersonList内的所有id为字符串
    let str = Object.values(allPersonList).flat().join(',');
    onChange(str);
    // setDataStatus()
  };
  const onCancel = () => {
    setVisible(false);
    setSearchVal('');
    // setDataStatus()
  };
  const onInput = e => {
    onSearch(e.target.value);
  };
  const onSearch = e => {
    const arr = [];
    setSearchVal(e);
    defaultPerson.map(item => {
      const list = {
        ouid: item.ouid,
        ouName: item.ouName,
        userList: [],
      };
      item.userList.map(user => {
        if (user.userName.indexOf(e) > -1) {
          list.userList.push(user);
        }
      });
      if (list.userList.length) {
        arr.push(list);
      }
    });
    setPersonArr(arr);
  };
  // 全选
  const all = (e) => {
    let _value = e.target.checked;
    setAllState(_value);
    let _allPerson = { ...allPersonList };
    let _checkAll = { ...checkAll };
    let _personArr = [...personArr];
    _personArr.forEach((item, index) => {
      _checkAll[`person_${index}`] = _value;
      _allPerson[`person_${index}`] = _value ? item.userList.map(obj => obj[storeType]) : []
    });
    setCheckAll(_checkAll);
    setAllPersonList(_allPerson);
  };
  // 复选框的函数
  const onCheckChange = (list, key) => {
    let _allPersonList = { ...allPersonList };
    _allPersonList[key] = list;
    setAllPersonList(_allPersonList);
  };
  const onCheckAllChange = (e, index) => {
    // 1. 设定选中状态
    let _value = e.target.checked;
    let _checkAll = { ...checkAll };
    _checkAll[`person_${index}`] = _value;
    setCheckAll(_checkAll);
    if (!_value) {
      let _allPersonList = { ...allPersonList };
      _allPersonList[`person_${index}`] = [];
      setAllPersonList(_allPersonList);
      setAllState(false)
      return false
    }
    // 2. 要把底下所有的值全部选中
    let _personArr = [...personArr];
    let _currentArray = _personArr[index];
    let _selectedValue = _currentArray.userList.map(item => item[storeType]);
    // 3. 赋值到allPersonList
    let _allPersonList = { ...allPersonList };
    _allPersonList[`person_${index}`] = _selectedValue;
    setAllPersonList(_allPersonList);
    let _isAll = true;
    Object.values(_checkAll).find(item => {
      if (!item) {
        _isAll = false;
      }
      return !item
    })
    if (_isAll) setAllState(true)
  };
  const checkSingleUser = (e, index) => {
    let _value = e.target.checked;
    let _checkAll = { ...checkAll };
    let _allPersonList = { ...allPersonList };
    let _personArr = [...personArr];
    if (!_value) {
      _checkAll[`person_${index}`] = false;
      setCheckAll(_checkAll);
      setAllState(false);
    } else {
      let _temp = Array.from(new Set(_allPersonList[`person_${index}`].concat(e.target.value)));
      if (_temp.length === _personArr[index].userList.length) {
        _checkAll[`person_${index}`] = true;
      }
      setCheckAll(_checkAll);
      let _isAll = true;
      Object.values(_checkAll).find(item => {
        if (!item) {
          _isAll = false;
        }
        return !item
      })
      if (_isAll) setAllState(true)
    }
  }

  let indeterminate = (Object.values(checkAll).some(v => v === false) && !Object.values(checkAll).every(v => v === false)) ||
    (Object.values(allPersonList).some(v => v.length) && Object.values(allPersonList).every(v => !v.length))

  return (
    <div className="personselect-wrapper" style={{ width: '100%' }}>
      <Input
        disabled={schema.disabled}
        value={userNames || null}
        placeholder={schema.disabled ? null : '点击选择人员'}
        addonAfter={!schema.disabled ? <UserAddOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={onFocus} /> : ''}
        style={{ width: '100%' }}
      />
      <Modal
        title="人员选择器"
        visible={visible}
        onOk={onOk}
        onCancel={onCancel}
        cancelText={'取消'}
        okText={'确定'}
        wrapClassName="personModalClass"
        bodyStyle={{ height: 450, overflowY: 'auto' }}
      >
        <Spin spinning={loading}>
          <Search
            placeholder="请输入姓名"
            value={searchVal}
            allowClear
            onSearch={onSearch}
            onChange={onInput}
            enterButton
          />
          <Checkbox
            style={{ marginTop: 18 }}
            onChange={(e) => all(e)}
            indeterminate={indeterminate}
            checked={allState}>
            全选
          </Checkbox>
          {personArr.length
            ? personArr.map((item, index) => {
              return (
                <form key={index} className="fieldset-class">
                  <fieldset className="person-select">
                    <legend className="personheader">
                      <Checkbox
                        style={{ marginRight: 8 }}
                        onChange={(e) => onCheckAllChange(e, index)}
                        indeterminate={Boolean(allPersonList[`person_${index}`] && allPersonList[`person_${index}`].length) && allPersonList[`person_${index}`].length !== item.userList.length}
                        checked={Boolean(allPersonList[`person_${index}`] && allPersonList[`person_${index}`].length)} />
                      {item.ouName}
                    </legend>
                    <Checkbox.Group className="person-radio" name="person"
                      onChange={(list) => onCheckChange(list, `person_${index}`)}
                      value={allPersonList[`person_${index}`]}>
                      {item.userList.map((child, childIndex) => {
                        return (
                          <Checkbox style={{ minWidth: 100 }}
                            value={child[storeType]} key={childIndex}
                            onChange={(e) => checkSingleUser(e, index)}>
                            {child.userName}
                          </Checkbox>
                        );
                      })}
                    </Checkbox.Group>
                  </fieldset>
                </form>
              );
            })
            :
            <div className={'staffSelectEmpty'}>
              <Empty />
            </div>
          }
        </Spin>
      </Modal>
    </div>
  );
};
export default MultipleStaffSelect;
