// 业务选择器
import React, { useState, useEffect, useMemo } from 'react';
import { Select } from 'antd';
import { GetFieldValueFromTable } from '../../../../apis/process';

const { Option } = Select;

const ServiceSelect = ({ value, onChange, name, schema }) => {

  const { disabled, placeholder } = schema
  const [selectList, setSelectList] = useState([]);

  const mode = useMemo(() => {
    if (schema.config.includes('n')) {
      return 'multiple'
    }
    return null
  }, [schema])

  const valueShow = useMemo(() => {
    let valueShow = value || schema.default
    if (valueShow.includes('请求错误')) {
      return mode ? '' : []
    }
    if (mode) {
      return Array.isArray(valueShow) ? valueShow : (valueShow ? valueShow.split(',') : [])
    } else {
      return valueShow
    }
  }, [schema, value])

  const handleChange = value => {
    if (mode) {
      onChange(value.join(','));
    } else {
      onChange(value)
    }
  };

  const getData = () => {
    const arr = schema.config.split('.');
    if (!arr.length) return false;
    const accountName = arr[0];
    const filedName = arr[1];
    GetFieldValueFromTable(accountName, filedName).then(res => {
      if (res.code === 0) {
        setSelectList(res.data || []);
      } else {
        onChange(`请求错误,${res.msg}`)
      }
    });
  };
  useEffect(() => {
    onChange(value || schema.default);
    getData()
  }, []);
  return (
    <Select
      mode={mode}
      disabled={disabled}
      showSearch
      showArrow={!disabled}
      optionFilterProp="children"
      value={valueShow || null}
      onChange={handleChange}
      placeholder={placeholder}>
      {selectList.map((item, index) => <Option value={item} key={index}>{item}</Option>)}
    </Select>
  );
};

export default ServiceSelect;
