// 站点选择器
import React, { useState, useEffect, useMemo } from 'react';
import { Select } from 'antd';
import { getStationListByUserID } from '../../../../apis/process';

const { Option } = Select;

const SiteSelect = ({ value, onChange, schema }) => {

  const { disabled, placeholder, config } = schema
  const [selectList, setSelectList] = useState([])
  const isMultiple = config && config.includes('.') && config.split('.')[1] === 'n'
  const userID = window?.globalConfig?.userInfo?.OID

  const valueShow = useMemo(() => {
    let val = value || schema.default
    if (isMultiple) {
      return val ? val.split(',') : []
    }
    return val ? val : null
  }, [value, schema])

  const handleChange = value => {
    if (isMultiple) {
      onChange(value.join(','))
    } else {
      onChange(value)
    }
  };

  const getData = () => {
    // 全部、全部.n，本人、本人.n，’‘
    // [id,sysType,isAll]
    let _params = [];
    if (!config || config.includes('全部')) {
      // 未配置，则默认全部
      _params = [userID];
    } else {
      _params = [userID, '', false];
    }
    getStationListByUserID(..._params).then(res => {
      if (res.code === 0) {
        setSelectList(res.data || []);
      } else {
        onChange('请求错误');
      }
    });
  };

  useEffect(() => {
    getData()
  }, [])

  return (
    <Select
      disabled={disabled}
      showSearch
      showArrow={!disabled}
      mode={isMultiple ? 'multiple' : ''}
      optionFilterProp="children"
      value={valueShow}
      onChange={handleChange}
      placeholder={placeholder}
    >
      {selectList.map((item, index) => <Option value={item} key={index}>{item}</Option>)}
    </Select>
  )
}

export default SiteSelect
