import React, { useMemo } from 'react'
import { Select } from 'antd'

const options = [
  {
    label: '邮箱(email)',
    value: '^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(.[a-zA-Z0-9_-]+)+$',
    message: '邮箱校验未通过'
  },
  {
    label: '身份证号(identity)',
    value: '(^\\d{15}$)|(^\\d{18}$)|(^\\d{17}(\\d|X|x)$)',
    message: '手机号校验未通过'
  },
  {
    label: '手机号(mobile)',
    value: '/^1(3\d|4[5-9]|5[0-35-9]|6[567]|7[0-8]|8\d|9[0-35-9])\d{8}$/',
    message: '手机号校验未通过',
  },
  {
    label: '银行卡号(bankAccount)',
    value: '^(\\d{16}|\\d{19})$',
    message: '银行卡号校验未通过'
  },
]

const VerifyTextInput = (props) => {

  const { value, onChange } = props

  const valueShow = useMemo(() => {
    return Array.isArray(value) && value.length ? value[0].pattern : null
  }, [value])

  const selectChange = (value) => {
    const item = options.find(v => v.value === value)

    onChange([{
      pattern: value,
      message: item.message,
    }])
  }

  return (
    <Select
      allowClear
      value={valueShow}
      options={options}
      onChange={selectChange}
    />
  )

}

export default VerifyTextInput